#!/usr/bin/perl -w

use strict;
use warnings;

########
#my $inFile                = "otherInfiles/infileNUM.txt";
my $inFile                 = "infile.txt";
my $outfile                = "outfile.txt";
my $transStartSite         = 16;
my $transStartSite_genomic = 45811288;

my $seedSpeciesName = "Mouse";

########

my $oneLineLength  = 60;
my $numEverySite   = 10;
my $maxNameLength  = 15;

open(IN,"$inFile") or die "Cannot find $inFile\n";
my @inFileCont = <IN>;
close IN;

my ($ref_names, $ref_seqs) = &fasHashReader2D(\@inFileCont);

my $ref_numTable           = &numCorresponder($seedSpeciesName, $transStartSite, $transStartSite_genomic, $ref_names, $ref_seqs);

($ref_names, $ref_seqs)    = &indexing($ref_numTable, $numEverySite, $ref_names, $ref_seqs);
#$ref_seqs                  = &matchFirst($seedSpeciesName, $ref_names, $ref_seqs);
$ref_names                 = &white2names($maxNameLength, $ref_names);

printing($outfile, $ref_numTable, $oneLineLength, $ref_names, $ref_seqs);


exit;
#####################



sub printing {
  my ($outfileSR, $refNumTableSR, $oneLineLengthSR, $ref_namesSR, $ref_seqsSR) = @_;
  my @namesSR      = @$ref_namesSR;
  my @secsSR       = @$ref_seqsSR;
  my @numTableSR   = @$refNumTableSR;
  my $speciesAmount = scalar(@namesSR);
  my $totalLengthSR = scalar(@numTableSR);

  open(OUT, ">$outfileSR");
  print OUT ($speciesAmount-1)." $totalLengthSR\n";
  print OUT "$numTableSR[0][1] $numTableSR[$totalLengthSR-1][1]\n";
  print OUT "$numTableSR[0][2] $numTableSR[$totalLengthSR-1][2]\n";

  my $lineLastSite = 0;
  for (my $num=0; $num<$totalLengthSR;) {
    if(($lineLastSite + $oneLineLengthSR) > $totalLengthSR){
      $lineLastSite = $totalLengthSR;
    }
    else{
      $lineLastSite += $oneLineLengthSR;
    }
  
    for (my $sp=0; $sp<$speciesAmount; $sp++) {

      my $spaciesName = $namesSR[$sp];
      if($spaciesName =~ /^GPOSIT/){
        my $gPositionTemp = &tSeparator($numTableSR[$num][2]);
        my $numChar = length($gPositionTemp);
        $spaciesName =~ s/GPOSIT/      /;
        $spaciesName =~ s/ {$numChar}/$gPositionTemp/;
      }
      print OUT "$spaciesName";

      for (my $linePosition=$num; $linePosition < $lineLastSite; $linePosition++){
        print OUT "$secsSR[$sp][$linePosition]";
      }
      print OUT "\n";
  
    }
  
    $num = $num + $oneLineLengthSR;
    print OUT "\n";
  }
  close OUT;
}


sub tSeparator{
  my ($numSR) = @_;
  1 while $numSR =~ s/(.*\d)(\d\d\d)/$1,$2/;
  return($numSR);
}


sub seedRecPicker{
  my($seedSpeciesNameSR, $ref_namesSR) = @_;
  my @namesSR = @$ref_namesSR;
  my $hitNum = "";
  for(my $i = 0; $i<scalar(@namesSR); $i++){
    if($namesSR[$i] eq ">".$seedSpeciesNameSR){
      $hitNum = $i;
      last;
    }
  }
  return ($hitNum);
}


sub numCorresponder {
  my ($seedSpeciesNameSR, $transStartSiteSR, $transStartSite_genomicSR, $ref_namesSR, $ref_seqsSR) = @_;
  my @seqsSR = @$ref_seqsSR;
  my $positionGenomic  = $transStartSite_genomicSR - $transStartSiteSR;
  my $positionRelative = -1 * $transStartSiteSR;

  my $referenceNum = &seedRecPicker($seedSpeciesNameSR, $ref_namesSR);
  my @numTableSR = ();
  my $totalLength = scalar(@{$seqsSR[$referenceNum]});
  for(my $i=0; $i<$totalLength; $i++){
    # print "Char $seqsSR[$referenceNum][$i], i $i, relative $positionRelative, genomic $positionGenomic\n";
    push(@numTableSR, [$seqsSR[$referenceNum][$i], $positionRelative, $positionGenomic]);
    if ($i+1 < $totalLength && $seqsSR[$referenceNum][$i] ne "-"){
      $positionGenomic++; $positionRelative++; 
    }
  }
  return(\@numTableSR);
}


sub num2charArray {
  my ($number) = @_;
  my $sign = "";
  if($number < 0){
    $sign = "-";
    $number = -1 * $number;
  }
  
  my @nums = ();
  if($number == 0) {
    @nums = ($number, @nums);
  } 
  elsif($number == 100){
    @nums = (1,0,0);
  }
  elsif($number == 10){
    @nums = (1,0);
  }
  else {
    while($number > 1){
      my $num = $number % 10;
      @nums = ($num, @nums);
      $number = $number/10;
    }
  }
  if($sign eq "-"){
    @nums = ("-", @nums);
  }
  return (\@nums);
}


sub indexing{

#  my($refNumTableSR, $numEverySiteSR, $refRecSR) = @_;
  my($refNumTableSR, $numEverySiteSR, $ref_namesSR, $ref_seqsSR) = @_;
  my @numTableSR = @$refNumTableSR;
  my $totalLength = scalar(@numTableSR);
  #my @recSR = @$refRecSR;
  my @namesSR = @$ref_namesSR;
  my @seqsSR  = @$ref_seqsSR;

  @namesSR = ('>GPOSIT', @namesSR);
  my @temp = ();
  for(my $j=0; $j<$totalLength; $j++){
    $temp[$j] = "_";
  }
  @seqsSR = (\@temp, @seqsSR);

  my $flag = 0;
  for(my $num=0; $num < $totalLength; ){
    if($numTableSR[$num][1] % $numEverySiteSR == 0){
      my $ref_numCharArray = &num2charArray($numTableSR[$num][1]);
      my @numChars = @$ref_numCharArray;
      if($flag == 0){
        for(my $k=0; $k<scalar(@numChars); $k++){
          #print "  a seqsSR[0][$num] = |$numChars[$k]|\n";
          $seqsSR[0][$num] = $numChars[$k];
          $num++;
        }
        $flag = 1;
      }
      else {
        #print "  b seqsSR[0][$num] = |_|\n";
        $seqsSR[0][$num] = "_";
        $num++;
      }
    } 
    else {
      $flag = 0 ;
      #print "  c seqsSR[0][$num] = |_|\n";
      $seqsSR[0][$num] = "_";
      $num++;
    }
  }

  return(\@namesSR, \@seqsSR);
}


sub white2names{
  my($maxNameLengthSR, $ref_namesSR)=@_;
  for my $ele (@$ref_namesSR) {
    $ele =~ s/^[<|>]//;
    my $white = " " x ($maxNameLength - length($ele));
    $ele = $ele.$white;
  }
  return($ref_namesSR);
}


sub matchFirst{
  my($seedSpeciesNameSR, $ref_namesSR, $ref_seqsSR)=@_;
  my @seqsSR = @$ref_seqsSR;
  my $speciesAmount  = scalar(@seqsSR)-1;
  my $totalLength    = scalar(@{$seqsSR[0]})-1;

  my $referenceNum = &seedRecPicker($seedSpeciesNameSR, $ref_namesSR);

  for (my $i=0; $i<$speciesAmount; $i++){
    next if $i == $referenceNum;
    for(my $j=0; $j<$totalLength; $j++){
      next if $seqsSR[$referenceNum][$j] =~ /[-NX]/;
      $seqsSR[$i][$j] = "." if $seqsSR[$i][$j] eq $seqsSR[$referenceNum][$j];
    }
  }
  return(\@seqsSR);
}


sub fasHashReader2D
{
  my ($inFileContSR) = @_;

  my $name = "";
  my $seq  = "";
  my $flag = 0;

  my @namesSR = ( );
  my @seqsSR  = ( );

  my $i = 0;
  foreach my $line (@$inFileContSR) {
    chomp($line);
    if($line =~ /^[<|>]/){  
      if($flag == 0){
        $name = $line;
        $flag = 1;  
      } else {
        $namesSR[$i] = $name;
        my @seqTemp = split(//,$seq);
        $seqsSR[$i]  = \@seqTemp;
        #print "seqsSR[$i] |@{$seqsSR[$i]}|\n";
        $i++;  

        $name = "$line";
        $seq = "";
      }
    } else {
      $seq .= $line; 
    }
  }
  $namesSR[$i] = $name;  
  my @seqTemp = split(//,$seq);
  $seqsSR[$i] = \@seqTemp;

  return (\@namesSR, \@seqsSR);
}
